@@ -124,11 +124,17 @@ gdk_window_copy_area_scroll (GdkWindow    *window,
   GdkWindowObject *obj = GDK_WINDOW_OBJECT (window);
   GList *tmp_list;
 
+  GDK_NOTE (MISC, g_print ("gdk_window_copy_area_scroll: %p %dx%d@+%d+%d %d,%d\n",
+			   GDK_WINDOW_HWND (window),
+			   dest_rect->width, dest_rect->height,
+			   dest_rect->x, dest_rect->y,
+			   dx, dy));
+
   if (dest_rect->width > 0 && dest_rect->height > 0)
     {
       RECT clipRect;
-      clipRect.left   = dest_rect->x - dx;
-      clipRect.top    = dest_rect->y - dy;
+      clipRect.left   = dest_rect->x;
+      clipRect.top    = dest_rect->y;
       clipRect.right  = clipRect.left + dest_rect->width;
       clipRect.bottom = clipRect.top + dest_rect->height;
 
@@ -137,13 +143,15 @@ gdk_window_copy_area_scroll (GdkWindow    *window,
       if (!ScrollWindowEx (GDK_WINDOW_HWND (window),
                            dx, dy, /* in: scroll offsets */
                            NULL, /* in: scroll rect, NULL == entire client area */
-                           &clipRect, /* in: restrict to, XXX: really want this ?? */
-                           NULL, /* out: update region */
+                           &clipRect, /* in: restrict to */
+                           NULL, /* in: update region */
                            NULL, /* out: update rect */
-                           SW_INVALIDATE))
+                           SW_INVALIDATE | SW_SCROLLCHILDREN))
         WIN32_API_FAILED ("ScrollWindowEx");
+      UpdateWindow (GDK_WINDOW_HWND (window));
     }
 
+#if 0 /* Not needed, ScrollWindowEx also scrolls the children. */
   tmp_list = obj->children;
   while (tmp_list)
     {
@@ -154,6 +162,7 @@ gdk_window_copy_area_scroll (GdkWindow    *window,
       
       tmp_list = tmp_list->next;
     }
+#endif
 }
 
 static void
@@ -202,6 +211,8 @@ compute_intermediate_position (GdkWin32PositionInfo *position_info,
   new_position->height = new_y1 - new_y0;
 }
 
+#if 0
+
 static void
 gdk_window_guffaw_scroll (GdkWindow    *window,
 			  gint          dx,
@@ -217,6 +228,9 @@ gdk_window_guffaw_scroll (GdkWindow    *window,
   GdkWindowParentPos parent_pos;
   GList *tmp_list;
   
+  GDK_NOTE (EVENTS, g_print ("gdk_window_guffaw_scroll: %p %d,%d\n",
+			     GDK_WINDOW_HWND (window), dx, dy));
+
   gdk_window_compute_parent_pos (impl, &parent_pos);
   gdk_window_compute_position (impl, &parent_pos, &new_info);
 
@@ -241,7 +255,7 @@ gdk_window_guffaw_scroll (GdkWindow    *window,
    */
   if (!SetWindowPos (GDK_WINDOW_HWND (window), NULL,
                      new_position.x, new_position.y,
-	               new_position.width, new_position.height,
+		     new_position.width, new_position.height,
                      SWP_NOACTIVATE | SWP_NOZORDER))
     WIN32_API_FAILED ("SetWindowPos");
   
@@ -282,6 +296,8 @@ gdk_window_guffaw_scroll (GdkWindow    *window,
     }
 }
 
+#endif
+
 /**
  * gdk_window_scroll:
  * @window: a #GdkWindow
@@ -302,7 +318,9 @@ gdk_window_scroll (GdkWindow *window,
 		   gint       dx,
 		   gint       dy)
 {
+#if 0
   gboolean can_guffaw_scroll = FALSE;
+#endif
   GdkRegion *invalidate_region;
   GdkWindowImplWin32 *impl;
   GdkWindowObject *obj;
@@ -313,6 +331,9 @@ gdk_window_scroll (GdkWindow *window,
   if (GDK_WINDOW_DESTROYED (window))
     return;
   
+  GDK_NOTE (EVENTS, g_print ("gdk_window_scroll: %p %d,%d\n",
+			     GDK_WINDOW_HWND (window), dx, dy));
+
   obj = GDK_WINDOW_OBJECT (window);
   impl = GDK_WINDOW_IMPL_WIN32 (obj->impl);  
 
@@ -341,7 +362,7 @@ gdk_window_scroll (GdkWindow *window,
   
   gdk_window_invalidate_region (window, invalidate_region, TRUE);
   gdk_region_destroy (invalidate_region);
-
+#if 0
   /* We can guffaw scroll if we are a child window, and the parent
    * does not extend beyond our edges. Otherwise, we use XCopyArea, then
    * move any children later
@@ -352,11 +373,13 @@ gdk_window_scroll (GdkWindow *window,
       can_guffaw_scroll = ((dx == 0 || (obj->x <= 0 && obj->x + impl->width >= parent_impl->width)) &&
 			   (dy == 0 || (obj->y <= 0 && obj->y + impl->height >= parent_impl->height)));
     }
-
   if (!obj->children || !can_guffaw_scroll)
     gdk_window_copy_area_scroll (window, &dest_rect, dx, dy);
   else
     gdk_window_guffaw_scroll (window, dx, dy);
+#else
+  gdk_window_copy_area_scroll (window, &dest_rect, dx, dy);
+#endif
 }
 
 void
@@ -371,6 +394,7 @@ _gdk_window_move_resize_child (GdkWindow *window,
   GdkWin32PositionInfo new_info;
   GdkWindowParentPos parent_pos;
   GList *tmp_list;
+  
   gint d_xoffset, d_yoffset;
   gint dx, dy;
   gboolean is_move;
@@ -379,9 +403,14 @@ _gdk_window_move_resize_child (GdkWindow *window,
   g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
 
+  impl = GDK_WINDOW_IMPL_WIN32 (GDK_WINDOW_OBJECT (window)->impl);
   obj = GDK_WINDOW_OBJECT (window);
-  impl = GDK_WINDOW_IMPL_WIN32 (obj->impl);
   
+  GDK_NOTE (MISC, g_print ("_gdk_window_move_resize_child: %s@+%d+%d %dx%d@+%d+%d\n",
+			   gdk_win32_drawable_description (window),
+			   obj->x, obj->y,
+			   width, height, x, y));
+
   dx = x - obj->x;
   dy = y - obj->y;
   
@@ -389,8 +418,13 @@ _gdk_window_move_resize_child (GdkWindow *window,
   is_resize = impl->width != width || impl->height != height;
 
   if (!is_move && !is_resize)
-    return;
+    {
+      GDK_NOTE (MISC, g_print ("...neither move or resize\n"));
+      return;
+    }
   
+  GDK_NOTE (MISC, g_print ("...%s%s\n", is_move ? "is_move " : "", is_resize ? "is_resize" : ""));
+
   obj->x = x;
   obj->y = y;
   impl->width = width;
@@ -414,6 +448,8 @@ _gdk_window_move_resize_child (GdkWindow *window,
     {
       GdkRectangle new_position;
 
+      GDK_NOTE (MISC, g_print ("...d_xoffset=%d d_yoffset=%d\n", d_xoffset, d_yoffset));
+
       gdk_window_set_static_gravities (window, TRUE);
 
       if (d_xoffset < 0 || d_yoffset < 0)
@@ -422,6 +458,10 @@ _gdk_window_move_resize_child (GdkWindow *window,
       compute_intermediate_position (&impl->position_info, &new_info, d_xoffset, d_yoffset,
 				     &new_position);
       
+      GDK_NOTE (MISC, g_print ("...SetWindowPos(%p,%dx%d@+%d+%d)\n",
+			       GDK_WINDOW_HWND (window),
+			       new_position.width, new_position.height,
+			       new_position.x, new_position.y));
       if (!SetWindowPos (GDK_WINDOW_HWND (window), NULL,
                          new_position.x, new_position.y, 
                          new_position.width, new_position.height,
@@ -435,6 +475,9 @@ _gdk_window_move_resize_child (GdkWindow *window,
 	  tmp_list = tmp_list->next;
 	}
 
+      GDK_NOTE (MISC, g_print ("...SetWindowPos(%p,0x0@+%d+%d)\n",
+			       GDK_WINDOW_HWND (window),
+			       new_position.x + dx, new_position.y + dy));
       if (!SetWindowPos (GDK_WINDOW_HWND (window), NULL,
                          new_position.x + dx, new_position.y + dy, 0, 0,
                          SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOSIZE | SWP_NOREDRAW))
@@ -443,6 +486,10 @@ _gdk_window_move_resize_child (GdkWindow *window,
       if (d_xoffset > 0 || d_yoffset > 0)
 	gdk_window_queue_translation (window, MAX (d_xoffset, 0), MAX (d_yoffset, 0));
       
+      GDK_NOTE (MISC, g_print ("...SetWindowPos(%p,%dx%d@+%d+%d)\n",
+			       GDK_WINDOW_HWND (window),
+			       new_info.width, new_info.height,
+			       new_info.x, new_info.y));
       if (!SetWindowPos (GDK_WINDOW_HWND (window), NULL,
                          new_info.x, new_info.y, 
                          new_info.width, new_info.height,
@@ -453,7 +500,11 @@ _gdk_window_move_resize_child (GdkWindow *window,
 	gdk_window_tmp_reset_bg (window);
 
       if (!impl->position_info.mapped && new_info.mapped && GDK_WINDOW_IS_MAPPED (obj))
-	ShowWindow (GDK_WINDOW_HWND (window), SW_SHOWNA);
+	{
+	  GDK_NOTE (MISC, g_print ("...ShowWindow(%p, SW_SHOWNA)\n",
+				   GDK_WINDOW_HWND (window)));
+	  ShowWindow (GDK_WINDOW_HWND (window), SW_SHOWNA);
+	}
       
       impl->position_info = new_info;
       
@@ -470,7 +521,11 @@ _gdk_window_move_resize_child (GdkWindow *window,
 	gdk_window_set_static_gravities (window, FALSE);
 
       if (impl->position_info.mapped && !new_info.mapped)
-	ShowWindow (GDK_WINDOW_HWND (window), SW_HIDE);
+	{
+	  GDK_NOTE (MISC, g_print ("...ShowWindow(%p, SW_HIDE)\n",
+				   GDK_WINDOW_HWND (window)));
+	  ShowWindow (GDK_WINDOW_HWND (window), SW_HIDE);
+	}
       
       tmp_list = obj->children;
       while (tmp_list)
@@ -479,13 +534,20 @@ _gdk_window_move_resize_child (GdkWindow *window,
 	  tmp_list = tmp_list->next;
 	}
 
+      GDK_NOTE (MISC, g_print ("...SetWindowPos(%p,%dx%d@+%d+%d)\n",
+			       GDK_WINDOW_HWND (window),
+			       new_info.width, new_info.height,
+			       new_info.x, new_info.y));
       if (!SetWindowPos (GDK_WINDOW_HWND (window), NULL,
                          new_info.x, new_info.y, 
                          new_info.width, new_info.height,
                          SWP_NOACTIVATE | SWP_NOZORDER | 
+                         (is_move ? 0 : SWP_NOMOVE) |
                          (is_resize ? 0 : SWP_NOSIZE)))
         WIN32_API_FAILED ("SetWindowPos");
 
+      UpdateWindow (GDK_WINDOW_HWND (window));
+
       tmp_list = obj->children;
       while (tmp_list)
 	{
@@ -497,7 +559,11 @@ _gdk_window_move_resize_child (GdkWindow *window,
 	gdk_window_tmp_reset_bg (window);
 
       if (!impl->position_info.mapped && new_info.mapped && GDK_WINDOW_IS_MAPPED (obj))
-	ShowWindow (GDK_WINDOW_HWND (window), SW_SHOWNA);
+	{
+	  GDK_NOTE (MISC, g_print ("...ShowWindow(%p, SW_SHOWNA)\n",
+				   GDK_WINDOW_HWND (window)));
+	  ShowWindow (GDK_WINDOW_HWND (window), SW_SHOWNA);
+	}
 
       impl->position_info = new_info;
     }
@@ -530,9 +596,9 @@ gdk_window_compute_position (GdkWindowImplWin32   *window,
       if (parent_pos->x + wrapper->x < -(SIZE_LIMIT/2))
 	{
 	  if (parent_pos->x + wrapper->x + window->width < (SIZE_LIMIT/2))
-	    info->x = parent_pos->x + wrapper->x + window->width - SIZE_LIMIT - parent_pos->win32_x;
+	    info->x = parent_pos->x + wrapper->x + window->width - info->width - parent_pos->win32_x;
 	  else
-	    info->x = -(SIZE_LIMIT/2) - parent_pos->win32_y;
+	    info->x = -(SIZE_LIMIT/2) - parent_pos->win32_x;
 	}
       else
 	info->x = parent_pos->x + wrapper->x - parent_pos->win32_x;
@@ -550,7 +616,7 @@ gdk_window_compute_position (GdkWindowImplWin32   *window,
       if (parent_pos->y + wrapper->y < -(SIZE_LIMIT/2))
 	{
 	  if (parent_pos->y + wrapper->y + window->height < (SIZE_LIMIT/2))
-	    info->y = parent_pos->y + wrapper->y + window->height - SIZE_LIMIT - parent_pos->win32_y;
+	    info->y = parent_pos->y + wrapper->y + window->height - info->height - parent_pos->win32_y;
 	  else
 	    info->y = -(SIZE_LIMIT/2) - parent_pos->win32_y;
 	}
@@ -614,7 +680,8 @@ gdk_window_compute_parent_pos (GdkWindowImplWin32      *window,
 
   g_return_if_fail (GDK_IS_WINDOW_IMPL_WIN32 (window));
 
-  wrapper = GDK_WINDOW_OBJECT (GDK_DRAWABLE_IMPL_WIN32 (window)->wrapper);
+  wrapper =
+    GDK_WINDOW_OBJECT (GDK_DRAWABLE_IMPL_WIN32 (window)->wrapper);
   
   parent_pos->x = 0;
   parent_pos->y = 0;
@@ -769,25 +836,51 @@ gdk_window_postmove (GdkWindow          *window,
     }
 }
 
+static void
+gdk_window_queue_append (GdkWindow          *window,
+			 GdkWindowQueueItem *item)
+{
+  if (g_slist_length (translate_queue) >= 128)
+    {
+      GdkWindowImplWin32 *impl = GDK_WINDOW_IMPL_WIN32 (GDK_WINDOW_OBJECT (window)->impl);
+
+      GDK_NOTE (EVENTS, g_print ("gdk_window_queue_append: length >= 128\n"));
+      _gdk_window_process_expose (window, _gdk_win32_get_next_tick (0),
+				  &impl->position_info.clip_rect);
+    }
+
+  item->window = window;
+  item->serial = GetTickCount ();
+
+  GDK_NOTE (EVENTS, (g_print ("gdk_window_queue_append: %s %p %ld ",
+			      (item->type == GDK_WINDOW_QUEUE_TRANSLATE ?
+			       "TRANSLATE" : "ANTIEXPOSE"),
+			      GDK_WINDOW_HWND (window),
+			      item->serial),
+		     (item->type == GDK_WINDOW_QUEUE_TRANSLATE ?
+		      g_print ("%d,%d\n",
+			       item->u.translate.dx, item->u.translate.dy) :
+		      g_print ("(%d,%d)--(%d,%d)\n",
+			       item->u.antiexpose.area->extents.x1,
+			       item->u.antiexpose.area->extents.y1,
+			       item->u.antiexpose.area->extents.x2,
+			       item->u.antiexpose.area->extents.y2))));
+
+  gdk_drawable_ref (window);
+  translate_queue = g_slist_append (translate_queue, item) ;
+}
+
 static void
 gdk_window_queue_translation (GdkWindow *window,
 			      gint       dx,
 			      gint       dy)
 {
   GdkWindowQueueItem *item = g_new (GdkWindowQueueItem, 1);
-  item->window = window;
-  item->serial = GetMessageTime ();
   item->type = GDK_WINDOW_QUEUE_TRANSLATE;
   item->u.translate.dx = dx;
   item->u.translate.dy = dy;
 
-  GDK_NOTE (EVENTS, g_print ("gdk_window_queue_translation %#x %ld %d,%d\n",
-			     (guint) GDK_WINDOW_HWND (window),
-			     item->serial,
-			     dx, dy));
-
-  gdk_drawable_ref (window);
-  translate_queue = g_slist_append (translate_queue, item);
+  gdk_window_queue_append (window, item);
 }
 
 gboolean
@@ -795,41 +888,15 @@ _gdk_windowing_window_queue_antiexpose (GdkWindow *window,
 					GdkRegion *area)
 {
 #if 1
+  return FALSE;			/* XXX */
+#else
   GdkWindowQueueItem *item = g_new (GdkWindowQueueItem, 1);
 
-  item->window = window;
-  item->serial = GetMessageTime ();
   item->type = GDK_WINDOW_QUEUE_ANTIEXPOSE;
   item->u.antiexpose.area = area;
 
-  GDK_NOTE (EVENTS, g_print ("_gdk_windowing_window_queue_antiexpose %#x %ld %dx%d@+%d+%d\n",
-			     (guint) GDK_WINDOW_HWND (window),
-			     item->serial,
-			     area->extents.x2 - area->extents.x1,
-			     area->extents.y2 - area->extents.y1,
-			     area->extents.x1, area->extents.y1));
+  gdk_window_queue_append (window, item);
 
-  gdk_drawable_ref (window);
-  translate_queue = g_slist_append (translate_queue, item);
-
-  return TRUE;
-#else
-  GdkRectangle r;
-  HRGN hrgn;
-
-  gdk_region_get_clipbox (area, &r);
-  hrgn = CreateRectRgn(r.x, r.y, r.width+1, r.height+1);
-
-  g_return_val_if_fail (area != NULL, FALSE);
-
-  GDK_NOTE (MISC, g_print ("_gdk_windowing_window_queue_antiexpose %#x\n",
-			   (guint) GDK_WINDOW_HWND (window)));
-
-  /* HB: not quite sure if this is the right thing to do.
-   * (Region not to be proceesed by next WM_PAINT)
-   */
-  ValidateRgn(GDK_WINDOW_HWND (window), hrgn);
-  DeleteObject(hrgn);
   return TRUE;
 #endif
 }
@@ -846,8 +913,8 @@ _gdk_window_process_expose (GdkWindow    *window,
   
   impl = GDK_WINDOW_IMPL_WIN32 (GDK_WINDOW_OBJECT (window)->impl);
   
-  GDK_NOTE (EVENTS, g_print ("_gdk_window_process_expose %#x %ld %dx%d@+%d+%d\n",
-			     (guint) GDK_WINDOW_HWND (window), serial,
+  GDK_NOTE (EVENTS, g_print ("_gdk_window_process_expose: %p %ld %dx%d@+%d+%d\n",
+			     GDK_WINDOW_HWND (window), serial,
 			     area->width, area->height, area->x, area->y));
   
   while (tmp_list)
@@ -860,15 +927,34 @@ _gdk_window_process_expose (GdkWindow    *window,
 	  if (item->window == window)
 	    {
 	      if (item->type == GDK_WINDOW_QUEUE_TRANSLATE)
-		gdk_region_offset (invalidate_region, - item->u.translate.dx, - item->u.translate.dy);
+		{
+		  GDK_NOTE (EVENTS, g_print ("...item %ld xlating region by %d,%d\n",
+					     item->serial,
+					     item->u.translate.dx, item->u.translate.dy));
+		  gdk_region_offset (invalidate_region, - item->u.translate.dx, - item->u.translate.dy);
+		}
 	      else		/* anti-expose */
-		gdk_region_subtract (invalidate_region, item->u.antiexpose.area);
+		{
+#ifdef G_ENABLE_DEBUG
+		  GdkRectangle rect;
+
+		  GDK_NOTE (EVENTS,
+			    (gdk_region_get_clipbox (item->u.antiexpose.area, &rect),
+			     g_print ("...item %ld antiexposing %dx%d@+%d+%d\n",
+				      item->serial,
+				      rect.width, rect.height,
+				      rect.x, rect.y)));
+#endif
+		  gdk_region_subtract (invalidate_region, item->u.antiexpose.area);
+		}
 	    }
 	}
       else
 	{
 	  GSList *tmp_link = translate_queue;
 	  
+	  GDK_NOTE (EVENTS, g_print ("...item %ld being removed\n", item->serial));
+
 	  translate_queue = g_slist_remove_link (translate_queue, translate_queue);
 	  gdk_drawable_unref (item->window);
 
@@ -880,6 +966,8 @@ _gdk_window_process_expose (GdkWindow    *window,
 	}
     }
 
+  GDK_NOTE (EVENTS, g_print ("...queue length now %d\n", g_slist_length (translate_queue)));
+
   clip_region = gdk_region_rectangle (&impl->position_info.clip_rect);
   gdk_region_intersect (invalidate_region, clip_region);
 
@@ -934,7 +1022,9 @@ gdk_window_tmp_reset_bg (GdkWindow *window)
 }
 
 static void
-gdk_window_clip_changed (GdkWindow *window, GdkRectangle *old_clip, GdkRectangle *new_clip)
+gdk_window_clip_changed (GdkWindow    *window,
+			 GdkRectangle *old_clip,
+			 GdkRectangle *new_clip)
 {
   GdkWindowImplWin32 *impl;
   GdkWindowObject *obj;
