@@ -25,15 +25,11 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include "config.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 
 #include "gdkpixmap.h"
-#include "gdkdisplay.h"
-#include "gdkscreen.h"
-
 #include "gdkprivate-win32.h"
 
 static void gdk_pixmap_impl_win32_get_size   (GdkDrawable        *drawable,
@@ -53,7 +49,7 @@ _gdk_pixmap_impl_win32_get_type (void)
 
   if (!object_type)
     {
-      const GTypeInfo object_info =
+      static const GTypeInfo object_info =
       {
         sizeof (GdkPixmapImplWin32Class),
         (GBaseInitFunc) NULL,
@@ -105,16 +101,19 @@ gdk_pixmap_impl_win32_finalize (GObject *object)
 {
   GdkPixmapImplWin32 *impl = GDK_PIXMAP_IMPL_WIN32 (object);
   GdkPixmap *wrapper = GDK_PIXMAP (GDK_DRAWABLE_IMPL_WIN32 (impl)->wrapper);
+  GdkImage *image = impl->image;
 
   GDK_NOTE (PIXMAP, g_print ("gdk_pixmap_impl_win32_finalize: %p\n",
 			     GDK_PIXMAP_HBITMAP (wrapper)));
 
-  _gdk_win32_drawable_finish (GDK_DRAWABLE (object));  
-
-  GDI_CALL (DeleteObject, (GDK_PIXMAP_HBITMAP (wrapper)));
+  if (!DeleteObject (GDK_PIXMAP_HBITMAP (wrapper)))
+    WIN32_GDI_FAILED ("DeleteObject");
 
   gdk_win32_handle_table_remove (GDK_PIXMAP_HBITMAP (wrapper));
 
+  image->windowing_data = NULL;
+  g_object_unref (image);
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -130,10 +129,11 @@ gdk_pixmap_impl_win32_get_size (GdkDrawable *drawable,
 }
 
 GdkPixmap*
-_gdk_pixmap_new (GdkDrawable *drawable,
-		gint         width,
-		gint         height,
-		gint         depth)
+_gdk_win32_pixmap_new (GdkWindow *window,
+		       GdkVisual *visual,
+		       gint       width,
+		       gint       height,
+		       gint       depth)
 {
   struct {
     BITMAPINFOHEADER bmiHeader;
@@ -145,33 +145,38 @@ _gdk_pixmap_new (GdkDrawable *drawable,
   } bmi;
   UINT iUsage;
   HDC hdc;
-  HWND hwnd;
   HPALETTE holdpal = NULL;
   HBITMAP hbitmap;
   GdkPixmap *pixmap;
   GdkDrawableImplWin32 *drawable_impl;
   GdkPixmapImplWin32 *pixmap_impl;
-  GdkColormap *cmap;
   guchar *bits;
   gint i;
-  gint window_depth;
 
-  g_return_val_if_fail (drawable == NULL || GDK_IS_DRAWABLE (drawable), NULL);
-  g_return_val_if_fail ((drawable != NULL) || (depth != -1), NULL);
+  g_return_val_if_fail (window == NULL || GDK_IS_WINDOW (window), NULL);
+  g_return_val_if_fail ((window != NULL) || (depth != -1), NULL);
   g_return_val_if_fail ((width != 0) && (height != 0), NULL);
 
-  if (!drawable)
-    drawable = _gdk_root;
+  if (!window)
+    window = _gdk_parent_root;
 
-  if (GDK_IS_WINDOW (drawable) && GDK_WINDOW_DESTROYED (drawable))
+  if (GDK_WINDOW_DESTROYED (window))
     return NULL;
 
-  window_depth = gdk_drawable_get_depth (GDK_DRAWABLE (drawable));
-  if (depth == -1)
-    depth = window_depth;
+  if (!visual)
+    {
+      if (window)
+	visual = gdk_drawable_get_visual (window);
+      else
+	visual = gdk_visual_get_system ();
+    }
 
-  GDK_NOTE (PIXMAP, g_print ("gdk_pixmap_new: %dx%dx%d drawable=%p\n",
-			     width, height, depth, drawable));
+  if (depth == -1)
+    depth = visual->depth;
+
+  GDK_NOTE (PIXMAP, g_print ("_gdk_win32_pixmap_new: %dx%dx%d "
+			     "window=%p visual=%p\n",
+			     width, height, depth, window, visual));
 
   pixmap = g_object_new (gdk_pixmap_get_type (), NULL);
   drawable_impl = GDK_DRAWABLE_IMPL_WIN32 (GDK_PIXMAP_OBJECT (pixmap)->impl);
@@ -183,18 +188,7 @@ _gdk_pixmap_new (GdkDrawable *drawable,
   pixmap_impl->height = height;
   GDK_PIXMAP_OBJECT (pixmap)->depth = depth;
 
-  if (depth == window_depth)
-    {
-      cmap = gdk_drawable_get_colormap (drawable);
-      if (cmap)
-        gdk_drawable_set_colormap (pixmap, cmap);
-    }
-  
-  if (GDK_IS_WINDOW (drawable))
-    hwnd = GDK_WINDOW_HWND (drawable);
-  else
-    hwnd = GetDesktopWindow ();
-  if ((hdc = GetDC (hwnd)) == NULL)
+  if ((hdc = GetDC (GDK_WINDOW_HWND (window))) == NULL)
     {
       WIN32_GDI_FAILED ("GetDC");
       g_object_unref ((GObject *) pixmap);
@@ -210,7 +204,7 @@ _gdk_pixmap_new (GdkDrawable *drawable,
     case 1:
     case 24:
     case 32:
-      bmi.bmiHeader.biBitCount = _gdk_windowing_get_bits_for_depth (gdk_display_get_default (), depth);
+      bmi.bmiHeader.biBitCount = depth;
       break;
 
     case 4:
@@ -257,10 +251,21 @@ _gdk_pixmap_new (GdkDrawable *drawable,
 	bmi.u.bmiColors[1].rgbGreen =
 	bmi.u.bmiColors[1].rgbRed = 0xFF;
       bmi.u.bmiColors[1].rgbReserved = 0x00;
+      drawable_impl->colormap = NULL;
     }
   else
     {
-      if (depth <= 8 && drawable_impl->colormap != NULL)
+      if (depth > 8 && depth != visual->depth)
+	g_warning ("_gdk_win32_pixmap_new: depth %d doesn't match display depth %d",
+		   depth, visual->depth);
+
+      drawable_impl->colormap = GDK_DRAWABLE_IMPL_WIN32 (GDK_WINDOW_OBJECT (window)->impl)->colormap;
+
+      if (drawable_impl->colormap == NULL)
+	drawable_impl->colormap = gdk_colormap_get_system ();
+      gdk_colormap_ref (drawable_impl->colormap);
+
+      if (depth <= 8)
 	{
 	  GdkColormapPrivateWin32 *cmapp =
 	    GDK_WIN32_COLORMAP_DATA (drawable_impl->colormap);
@@ -280,8 +285,6 @@ _gdk_pixmap_new (GdkDrawable *drawable,
 	}
       else if (bmi.bmiHeader.biBitCount == 16)
 	{
-	  GdkVisual *visual = gdk_visual_get_system ();
-
 	  bmi.u.bmiMasks[0] = visual->red_mask;
 	  bmi.u.bmiMasks[1] = visual->green_mask;
 	  bmi.u.bmiMasks[2] = visual->blue_mask;
@@ -293,26 +296,39 @@ _gdk_pixmap_new (GdkDrawable *drawable,
   if (holdpal != NULL)
     SelectPalette (hdc, holdpal, FALSE);
 
-  GDI_CALL (ReleaseDC, (hwnd, hdc));
+  if (!ReleaseDC (GDK_WINDOW_HWND (window), hdc))
+    WIN32_GDI_FAILED ("ReleaseDC");
 
-  GDK_NOTE (PIXMAP, g_print ("... =%p bits=%p pixmap=%p\n", hbitmap, bits, pixmap));
+  GDK_NOTE (PIXMAP, g_print ("...=%p bits=%p\n", hbitmap, bits));
 
   if (hbitmap == NULL)
     {
       WIN32_GDI_FAILED ("CreateDIBSection");
+      ReleaseDC (GDK_WINDOW_HWND (window), hdc);
       g_object_unref ((GObject *) pixmap);
       return NULL;
     }
 
   drawable_impl->handle = hbitmap;
-  pixmap_impl->bits = bits;
+  pixmap_impl->image = _gdk_win32_setup_pixmap_image (pixmap, window,
+						      width, height,
+						      depth, bits);
 
   gdk_win32_handle_table_insert (&GDK_PIXMAP_HBITMAP (pixmap), pixmap);
 
   return pixmap;
 }
 
-static const unsigned char mirror[256] = {
+GdkPixmap*
+gdk_pixmap_new (GdkWindow *window,
+		gint       width,
+		gint       height,
+		gint       depth)
+{
+  return _gdk_win32_pixmap_new (window, NULL, width, height, depth);
+}
+
+static unsigned char mirror[256] = {
   0x00, 0x80, 0x40, 0xc0, 0x20, 0xa0, 0x60, 0xe0,
   0x10, 0x90, 0x50, 0xd0, 0x30, 0xb0, 0x70, 0xf0,
   0x08, 0x88, 0x48, 0xc8, 0x28, 0xa8, 0x68, 0xe8,
@@ -348,38 +364,39 @@ static const unsigned char mirror[256] = {
 };
 
 GdkPixmap *
-_gdk_bitmap_create_from_data (GdkDrawable *drawable,
+gdk_bitmap_create_from_data (GdkWindow   *window,
 			     const gchar *data,
 			     gint         width,
 			     gint         height)
 {
   GdkPixmap *pixmap;
   GdkPixmapImplWin32 *pixmap_impl;
-  gint i, j, data_bpl, pixmap_bpl;
+  gint i, j, data_bpl, image_bpl;
   guchar *bits;
 
   g_return_val_if_fail (data != NULL, NULL);
   g_return_val_if_fail ((width != 0) && (height != 0), NULL);
-  g_return_val_if_fail (drawable == NULL || GDK_IS_DRAWABLE (drawable), NULL);
+  g_return_val_if_fail (window == NULL || GDK_IS_WINDOW (window), NULL);
 
-  if (!drawable)
-    drawable = _gdk_root;
-  else if (GDK_IS_WINDOW (drawable) && GDK_WINDOW_DESTROYED (drawable))
+  if (!window)
+    window = _gdk_parent_root;
+
+  if (GDK_WINDOW_DESTROYED (window))
     return NULL;
 
-  pixmap = gdk_pixmap_new (drawable, width, height, 1);
+  pixmap = gdk_pixmap_new (window, width, height, 1);
 
   if (pixmap == NULL)
     return NULL;
 
   pixmap_impl = GDK_PIXMAP_IMPL_WIN32 (GDK_PIXMAP_OBJECT (pixmap)->impl);
-  bits = pixmap_impl->bits;
+  bits = pixmap_impl->image->mem;
   data_bpl = ((width - 1) / 8 + 1);
-  pixmap_bpl = ((width - 1)/32 + 1)*4;
+  image_bpl = pixmap_impl->image->bpl;
 
   for (i = 0; i < height; i++)
     for (j = 0; j < data_bpl; j++)
-      bits[i*pixmap_bpl + j] = mirror[(guchar) data[i*data_bpl + j]];
+      bits[i*image_bpl + j] = mirror[(guchar) data[i*data_bpl + j]];
 
   GDK_NOTE (PIXMAP, g_print ("gdk_bitmap_create_from_data: %dx%d=%p\n",
 			     width, height, GDK_PIXMAP_HBITMAP (pixmap)));
@@ -388,13 +405,13 @@ _gdk_bitmap_create_from_data (GdkDrawable *drawable,
 }
 
 GdkPixmap*
-_gdk_pixmap_create_from_data (GdkDrawable    *drawable,
-			     const gchar    *data,
-			     gint            width,
-			     gint            height,
-			     gint            depth,
-			     const GdkColor *fg,
-			     const GdkColor *bg)
+gdk_pixmap_create_from_data (GdkWindow   *window,
+			     const gchar *data,
+			     gint         width,
+			     gint         height,
+			     gint         depth,
+			     GdkColor    *fg,
+			     GdkColor    *bg)
 {
   /* Oh wow. I struggled with dozens of lines of code trying to get
    * this right using a monochrome Win32 bitmap created from data, and
@@ -407,18 +424,11 @@ _gdk_pixmap_create_from_data (GdkDrawable    *drawable,
   GdkPixmap *source;
   GdkGC *gc;
 
-  g_return_val_if_fail (drawable == NULL || GDK_IS_DRAWABLE (drawable), NULL);
-  g_return_val_if_fail (data != NULL, NULL);
-  g_return_val_if_fail (fg != NULL, NULL);
-  g_return_val_if_fail (bg != NULL, NULL);
-  g_return_val_if_fail ((drawable != NULL) || (depth != -1), NULL);
-  g_return_val_if_fail ((width != 0) && (height != 0), NULL);
-
-  if (GDK_IS_WINDOW (drawable) && GDK_WINDOW_DESTROYED (drawable))
+  if (GDK_WINDOW_DESTROYED (window))
     return NULL;
 
-  result = gdk_pixmap_new (drawable, width, height, depth);
-  source = gdk_bitmap_create_from_data (drawable, data, width, height);
+  result = gdk_pixmap_new (window, width, height, depth);
+  source = gdk_bitmap_create_from_data (window, data, width, height);
   gc = gdk_gc_new (result);
 
   gdk_gc_set_foreground (gc, fg);
@@ -427,8 +437,8 @@ _gdk_pixmap_create_from_data (GdkDrawable    *drawable,
     (TRUE,
      GDK_DRAWABLE_IMPL_WIN32 (GDK_PIXMAP_OBJECT (result)->impl),
      gc, source, 0, 0, 0, 0, width, height);
-  g_object_unref (source);
-  g_object_unref (gc);
+  gdk_drawable_unref (source);
+  gdk_gc_unref (gc);
 
   GDK_NOTE (PIXMAP, g_print ("gdk_pixmap_create_from_data: %dx%dx%d=%p\n",
 			     width, height, depth,
@@ -437,28 +447,6 @@ _gdk_pixmap_create_from_data (GdkDrawable    *drawable,
   return result;
 }
 
-GdkPixmap *
-gdk_pixmap_foreign_new_for_display (GdkDisplay      *display,
-				    GdkNativeWindow  anid)
-{
-  g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
-  g_return_val_if_fail (display == _gdk_display, NULL);
-
-  return gdk_pixmap_foreign_new (anid);
-}
-
-GdkPixmap *
-gdk_pixmap_foreign_new_for_screen (GdkScreen       *screen,
-				   GdkNativeWindow  anid,
-				   gint             width,
-				   gint             height,
-				   gint             depth)
-{
-  g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
-
-  return gdk_pixmap_foreign_new (anid);
-}
-
 GdkPixmap*
 gdk_pixmap_foreign_new (GdkNativeWindow anid)
 {
@@ -488,24 +476,13 @@ gdk_pixmap_foreign_new (GdkNativeWindow anid)
   draw_impl->colormap = NULL;
   pix_impl->width = size.cx;
   pix_impl->height = size.cy;
-  pix_impl->bits = NULL;
+  pix_impl->image =
+    _gdk_win32_setup_pixmap_image (pixmap, _gdk_parent_root,
+				   size.cx, size.cy,
+				   gdk_visual_get_system ()->depth,
+				   NULL);
 
   gdk_win32_handle_table_insert (&GDK_PIXMAP_HBITMAP (pixmap), pixmap);
 
   return pixmap;
 }
-
-GdkPixmap*
-gdk_pixmap_lookup (GdkNativeWindow anid)
-{
-  return (GdkPixmap*) gdk_win32_handle_table_lookup (anid);
-}
-
-GdkPixmap*
-gdk_pixmap_lookup_for_display (GdkDisplay *display, GdkNativeWindow anid)
-{
-  g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
-  g_return_val_if_fail (display == _gdk_display, NULL);
-
-  return gdk_pixmap_lookup (anid);
-}
